﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  /// <summary>
  /// An operator for analyzing the best solution of Generalized Quadratic Assignment Problems.
  /// </summary>
  [Item("GeneralizedQAP Solution Archive Analyzer", "An operator for analyzing the archive of found solutions for the GQAP.")]
  [StorableClass]
  public sealed class GQAPSolutionArchiveAnalyzer : SingleSuccessorOperator, IAnalyzer {

    public bool EnabledByDefault {
      get { return true; }
    }

    public LookupParameter<BoolValue> MaximizationParameter {
      get { return (LookupParameter<BoolValue>)Parameters["Maximization"]; }
    }
    public LookupParameter<DoubleMatrix> DistancesParameter {
      get { return (LookupParameter<DoubleMatrix>)Parameters["Distances"]; }
    }
    public LookupParameter<DoubleMatrix> WeightsParameter {
      get { return (LookupParameter<DoubleMatrix>)Parameters["Weights"]; }
    }
    public LookupParameter<DoubleMatrix> InstallationCostsParameter {
      get { return (LookupParameter<DoubleMatrix>)Parameters["InstallationCosts"]; }
    }
    public LookupParameter<DoubleArray> DemandsParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["Demands"]; }
    }
    public LookupParameter<DoubleArray> CapacitiesParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["Capacities"]; }
    }
    public LookupParameter<DoubleValue> TransportationCostsParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["TransportationCosts"]; }
    }
    public LookupParameter<DoubleValue> OverbookedCapacityPenaltyParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["OverbookedCapacityPenalty"]; }
    }
    public ScopeTreeLookupParameter<IntegerVector> AssignmentParameter {
      get { return (ScopeTreeLookupParameter<IntegerVector>)Parameters["Assignment"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> QualityParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> FlowDistanceQualityParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["FlowDistanceQuality"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> InstallationQualityParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["InstallationQuality"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> OverbookedCapacityParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["OverbookedCapacity"]; }
    }
    public LookupParameter<GQAPAssignment> BestSolutionParameter {
      get { return (LookupParameter<GQAPAssignment>)Parameters["BestSolution"]; }
    }
    public ValueLookupParameter<ResultCollection> ResultsParameter {
      get { return (ValueLookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    public LookupParameter<DoubleValue> BestKnownQualityParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["BestKnownQuality"]; }
    }
    public LookupParameter<IntegerVector> BestKnownSolutionParameter {
      get { return (LookupParameter<IntegerVector>)Parameters["BestKnownSolution"]; }
    }
    public ILookupParameter<StringArray> EquipmentNamesParameter {
      get { return (ILookupParameter<StringArray>)Parameters["EquipmentNames"]; }
    }
    public ILookupParameter<StringArray> LocationNamesParameter {
      get { return (ILookupParameter<StringArray>)Parameters["LocationNames"]; }
    }

    [StorableConstructor]
    private GQAPSolutionArchiveAnalyzer(bool deserializing) : base(deserializing) { }
    private GQAPSolutionArchiveAnalyzer(GQAPSolutionArchiveAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GQAPSolutionArchiveAnalyzer(this, cloner);
    }
    public GQAPSolutionArchiveAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<BoolValue>("Maximization", "True if the problem is a maximization problem."));
      Parameters.Add(new LookupParameter<DoubleMatrix>("Distances", "The distances between the locations."));
      Parameters.Add(new LookupParameter<DoubleMatrix>("Weights", "The weights between the equipments."));
      Parameters.Add(new LookupParameter<DoubleMatrix>("InstallationCosts", "The cost of installing equipment x at location y."));
      Parameters.Add(new LookupParameter<DoubleArray>("Demands", "The demands of the equipments."));
      Parameters.Add(new LookupParameter<DoubleArray>("Capacities", "The capacities at the locations."));
      Parameters.Add(new LookupParameter<DoubleValue>("TransportationCosts", "The transportation cost represents the flow-unit per distance-unit cost factor. This value can also be set to 1 if these costs are factored into the weights matrix already."));
      Parameters.Add(new LookupParameter<DoubleValue>("OverbookedCapacityPenalty", "The multiplier for the constraint violation when added to the quality."));
      Parameters.Add(new ScopeTreeLookupParameter<IntegerVector>("Assignment", "The GQAP solutions from which the best solution should be analyzed."));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Quality", "The qualities of the GQAP solutions which should be analyzed."));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("FlowDistanceQuality", "The flow-distance qualities of the GQAP solutions which should be analyzed."));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("InstallationQuality", "The installation qualities of the GQAP solutions which should be analyzed."));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("OverbookedCapacity", "The overbooked capacities of the GQAP solutions which should be analyzed."));
      Parameters.Add(new LookupParameter<GQAPAssignment>("BestSolution", "The best GQAP solution."));
      Parameters.Add(new ValueLookupParameter<ResultCollection>("Results", "The result collection where the best GQAP solution should be stored."));
      Parameters.Add(new LookupParameter<DoubleValue>("BestKnownQuality", "The quality of the best known solution of this GQAP instance."));
      Parameters.Add(new LookupParameter<IntegerVector>("BestKnownSolution", "The best known solution of this GQAP instance."));
      Parameters.Add(new LookupParameter<StringArray>("EquipmentNames", "A list of names that describes the equipments."));
      Parameters.Add(new LookupParameter<StringArray>("LocationNames", "A list of names that describes the locations."));
    }

    public override IOperation Apply() {
      var assignments = AssignmentParameter.ActualValue;
      var qualities = QualityParameter.ActualValue;
      var equipmentNames = EquipmentNamesParameter.ActualValue;
      var locationNames = LocationNamesParameter.ActualValue;
      var flowDistanceQualities = FlowDistanceQualityParameter.ActualValue;
      var installationQualities = InstallationQualityParameter.ActualValue;
      var overbookedCapacities = OverbookedCapacityParameter.ActualValue;
      var distances = DistancesParameter.ActualValue;
      var weights = WeightsParameter.ActualValue;
      var installationCosts = InstallationCostsParameter.ActualValue;
      var demands = DemandsParameter.ActualValue;
      var capacities = CapacitiesParameter.ActualValue;
      var transportationCosts = TransportationCostsParameter.ActualValue;
      var overbookedCapacityPenalty = OverbookedCapacityPenaltyParameter.ActualValue;
      var results = ResultsParameter.ActualValue;
      var maximization = MaximizationParameter.ActualValue.Value;
      var bestKnownQuality = BestKnownQualityParameter.ActualValue;

      GQAPAssignmentArchive archive = results.ContainsKey("Solution Archive") ? results["Solution Archive"].Value as GQAPAssignmentArchive : null;
      if (archive == null) {
        archive = new GQAPAssignmentArchive(equipmentNames, locationNames, distances, weights, installationCosts, demands, capacities, transportationCosts, overbookedCapacityPenalty);
        results.Add(new Result("Solution Archive", archive));
      } else {
        archive.EquipmentNames = equipmentNames;
        archive.LocationNames = locationNames;
        archive.Distances = distances;
        archive.Weights = weights;
        archive.InstallationCosts = installationCosts;
        archive.Demands = demands;
        archive.Capacities = capacities;
        archive.TransportationCosts = transportationCosts;
        archive.OverbookedCapacityPenalty = overbookedCapacityPenalty;
      }

      ItemList<GQAPSolution> front = new ItemList<GQAPSolution>(archive.Solutions);
      for (int i = 0; i < assignments.Length; i++) {
        if (overbookedCapacities[i].Value <= 0.0)
          front.Add(new GQAPSolution(assignments[i], qualities[i], flowDistanceQualities[i], installationQualities[i], overbookedCapacities[i]));
      }

      for (int i = 0; i < front.Count - 1; i++) {
        for (int j = i + 1; j < front.Count; j++) {
          if (front[i].FlowDistanceQuality.Value < front[j].FlowDistanceQuality.Value
            && front[i].InstallationQuality.Value < front[j].InstallationQuality.Value) {
            front.RemoveAt(j);
            j--;
          } else if (front[i].FlowDistanceQuality.Value > front[j].FlowDistanceQuality.Value
            && front[i].InstallationQuality.Value > front[j].InstallationQuality.Value) {
            front.RemoveAt(i);
            j = i + 1;
          }
        }
      }

      archive.Solutions = front;

      return base.Apply();
    }
  }
}
